@@ -7,14 +7,37 @@ import numpy as np
 from ..utils import get_vectors
 from ._discrete_interpolator import DiscreteInterpolator
 from ..interpolators import InterpolatorType
-
-from ._operator import Operator
-
+from scipy.spatial import KDTree
 from LoopStructural.utils import getLogger
 
 logger = getLogger(__name__)
 
 
+def compute_weighting(grid_points, gradient_constraint_points, alpha=10.0, sigma=1.0):
+    """
+    Compute weights for second derivative regularization based on proximity to gradient constraints.
+
+    Parameters:
+        grid_points (ndarray): (N, 3) array of 3D coordinates for grid cells.
+        gradient_constraint_points (ndarray): (M, 3) array of 3D coordinates for gradient constraints.
+        alpha (float): Strength of weighting increase.
+        sigma (float): Decay parameter for Gaussian-like influence.
+
+    Returns:
+        weights (ndarray): (N,) array of weights for each grid point.
+    """
+    # Build a KDTree with the gradient constraint locations
+    tree = KDTree(gradient_constraint_points)
+
+    # Find the distance from each grid point to the nearest gradient constraint
+    distances, _ = tree.query(grid_points, k=1)
+
+    # Compute weighting function (higher weight for nearby points)
+    weights = 1 + alpha * np.exp(-(distances**2) / (2 * sigma**2))
+
+    return weights
+
+
 class FiniteDifferenceInterpolator(DiscreteInterpolator):
     def __init__(self, grid, data={}):
         """
@@ -26,13 +49,11 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         """
         self.shape = "rectangular"
         DiscreteInterpolator.__init__(self, grid, data=data)
-        # default weights for the interpolation matrix are 1 in x,y,z and
-        # 1/
         self.set_interpolation_weights(
             {
-                "dxy": 0.7,
-                "dyz": 0.7,
-                "dxz": 0.7,
+                "dxy": 1.0,
+                "dyz": 1.0,
+                "dxz": 1.0,
                 "dxx": 1.0,
                 "dyy": 1.0,
                 "dzz": 1.0,
@@ -47,8 +68,8 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
             }
         )
 
-        # grid.step_vector[2]
         self.type = InterpolatorType.FINITE_DIFFERENCE
+        self.use_regularisation_weight_scale = False
 
     def setup_interpolator(self, **kwargs):
         """
@@ -81,50 +102,32 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         for key in kwargs:
             self.up_to_date = False
             if "regularisation" in kwargs:
-                self.interpolation_weights["dxy"] = 0.1 * kwargs["regularisation"]
-                self.interpolation_weights["dyz"] = 0.1 * kwargs["regularisation"]
-                self.interpolation_weights["dxz"] = 0.1 * kwargs["regularisation"]
-                self.interpolation_weights["dxx"] = 0.1 * kwargs["regularisation"]
-                self.interpolation_weights["dyy"] = 0.1 * kwargs["regularisation"]
-                self.interpolation_weights["dzz"] = 0.1 * kwargs["regularisation"]
+                self.interpolation_weights["dxy"] = kwargs["regularisation"]
+                self.interpolation_weights["dyz"] = kwargs["regularisation"]
+                self.interpolation_weights["dxz"] = kwargs["regularisation"]
+                self.interpolation_weights["dxx"] = kwargs["regularisation"]
+                self.interpolation_weights["dyy"] = kwargs["regularisation"]
+                self.interpolation_weights["dzz"] = kwargs["regularisation"]
             self.interpolation_weights[key] = kwargs[key]
-        # if we want to define the operators manually
-        if "operators" in kwargs:
-            for o in kwargs["operators"].values():
-                self.assemble_inner(o[0], o[1])
-        # otherwise just use defaults
-        if "operators" not in kwargs:
-            operator = Operator.Dxy_mask
-            weight = (
-                self.interpolation_weights["dxy"] / 4
-            )  # (4*self.support.step_vector[0]*self.support.step_vector[1])
-            self.assemble_inner(operator, weight)
-            operator = Operator.Dyz_mask
-            weight = (
-                self.interpolation_weights["dyz"] / 4
-            )  # (4*self.support.step_vector[1]*self.support.step_vector[2])
-            self.assemble_inner(operator, weight)
-            operator = Operator.Dxz_mask
-            weight = (
-                self.interpolation_weights["dxz"] / 4
-            )  # (4*self.support.step_vector[0]*self.support.step_vector[2])
-            self.assemble_inner(operator, weight)
-            operator = Operator.Dxx_mask
-            weight = self.interpolation_weights["dxx"] / 1  # self.support.step_vector[0]**2
-            self.assemble_inner(operator, weight)
-            operator = Operator.Dyy_mask
-            weight = self.interpolation_weights["dyy"] / 1  # self.support.step_vector[1]**2
-            self.assemble_inner(operator, weight)
-            operator = Operator.Dzz_mask
-            weight = self.interpolation_weights["dzz"] / 1  # self.support.step_vector[2]**2
-            self.assemble_inner(operator, weight)
+        # either use the default operators or the ones passed to the function
+        operators = kwargs.get(
+            "operators", self.support.get_operators(weights=self.interpolation_weights)
+        )
+
+        self.use_regularisation_weight_scale = kwargs.get('use_regularisation_weight_scale', False)
         self.add_norm_constraints(self.interpolation_weights["npw"])
         self.add_gradient_constraints(self.interpolation_weights["gpw"])
         self.add_value_constraints(self.interpolation_weights["cpw"])
         self.add_tangent_constraints(self.interpolation_weights["tpw"])
         self.add_interface_constraints(self.interpolation_weights["ipw"])
         self.add_value_inequality_constraints()
-        self.add_inequality_pairs_constraints()
+        self.add_inequality_pairs_constraints(
+            pairs=kwargs.get('inequality_pairs', None),
+            upper_bound=kwargs.get('inequality_pair_upper_bound', np.finfo(float).eps),
+            lower_bound=kwargs.get('inequality_pair_lower_bound', -np.inf),
+        )
+        for k, o in operators.items():
+            self.assemble_inner(o[0], o[1], name=k)
 
     def copy(self):
         """
@@ -157,7 +160,7 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
             # print(points[inside,:].shape)
             gi = np.zeros(self.support.n_nodes, dtype=int)
             gi[:] = -1
-            gi[self.region] = np.arange(0, self.nx, dtype=int)
+            gi[self.region] = np.arange(0, self.dof, dtype=int)
             idc = np.zeros(node_idx.shape)
             idc[:] = -1
             idc[inside, :] = gi[node_idx[inside, :]]
@@ -167,9 +170,9 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
             # a/=self.support.enp.product(self.support.step_vector)
             self.add_constraints_to_least_squares(
                 a,
-                points[inside, 3],
+                points[inside, self.support.dimension],
                 idc[inside, :],
-                w=w * points[inside, 4],
+                w=w * points[inside, self.support.dimension + 1],
                 name="value",
             )
             if np.sum(inside) <= 0:
@@ -196,18 +199,25 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         # get elements for points
         points = self.get_interface_constraints()
         if points.shape[0] > 1:
-            vertices, c, tetras, inside = self.support.get_element_for_location(
+            node_idx, inside = self.support.position_to_cell_corners(
                 points[:, : self.support.dimension]
             )
-            # calculate volume of tetras
-            # vecs = vertices[inside, 1:, :] - vertices[inside, 0, None, :]
-            # vol = np.abs(np.linalg.det(vecs)) / 6
-            A = c[inside, :]
-            # A *= vol[:,None]
-            idc = tetras[inside, :]
-
-            for unique_id in np.unique(points[np.logical_and(~np.isnan(points[:, 3]), inside), 3]):
-                mask = points[inside, 3] == unique_id
+            gi = np.zeros(self.support.n_nodes, dtype=int)
+            gi[:] = -1
+            gi[self.region] = np.arange(0, self.dof, dtype=int)
+            idc = np.zeros(node_idx.shape).astype(int)
+            idc[:] = -1
+            idc[inside, :] = gi[node_idx[inside, :]]
+            inside = np.logical_and(~np.any(idc == -1, axis=1), inside)
+            idc = idc[inside, :]
+            A = self.support.position_to_dof_coefs(points[inside, : self.support.dimension])
+            for unique_id in np.unique(
+                points[
+                    np.logical_and(~np.isnan(points[:, self.support.dimension]), inside),
+                    self.support.dimension,
+                ]
+            ):
+                mask = points[inside, self.support.dimension] == unique_id
                 ij = np.array(
                     np.meshgrid(
                         np.arange(0, A[mask, :].shape[0]),
@@ -216,13 +226,12 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
                 ).T.reshape(-1, 2)
                 interface_A = np.hstack([A[mask, :][ij[:, 0], :], -A[mask, :][ij[:, 1], :]])
                 interface_idc = np.hstack([idc[mask, :][ij[:, 0], :], idc[mask, :][ij[:, 1], :]])
-
                 # now map the index from global to region create array size of mesh
-                # initialise as np.nan, then map points inside region to 0->nx
+                # initialise as np.nan, then map points inside region to 0->dof
                 gi = np.zeros(self.support.n_nodes).astype(int)
                 gi[:] = -1
 
-                gi[self.region] = np.arange(0, self.nx)
+                gi[self.region] = np.arange(0, self.dof)
                 interface_idc = gi[interface_idc]
                 outside = ~np.any(interface_idc == -1, axis=1)
                 self.add_constraints_to_least_squares(
@@ -248,15 +257,16 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         points = self.get_gradient_constraints()
         if points.shape[0] > 0:
             # calculate unit vector for orientation data
-            # points[:,3:]/=np.linalg.norm(points[:,3:],axis=1)[:,None]
 
-            node_idx, inside = self.support.position_to_cell_corners(points[:, :3])
+            node_idx, inside = self.support.position_to_cell_corners(
+                points[:, : self.support.dimension]
+            )
             # calculate unit vector for node gradients
             # this means we are only constraining direction of grad not the
             # magnitude
             gi = np.zeros(self.support.n_nodes)
             gi[:] = -1
-            gi[self.region] = np.arange(0, self.nx)
+            gi[self.region] = np.arange(0, self.dof)
             idc = np.zeros(node_idx.shape)
             idc[:] = -1
             idc[inside, :] = gi[node_idx[inside, :]]
@@ -267,18 +277,32 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
                 T,
                 elements,
                 inside_,
-            ) = self.support.get_element_gradient_for_location(points[inside, :3])
+            ) = self.support.get_element_gradient_for_location(
+                points[inside, : self.support.dimension]
+            )
             # normalise constraint vector and scale element matrix by this
-            norm = np.linalg.norm(points[:, 3:6], axis=1)
+            norm = np.linalg.norm(
+                points[:, self.support.dimension : self.support.dimension + self.support.dimension],
+                axis=1,
+            )
             points[:, 3:6] /= norm[:, None]
             T /= norm[inside, None, None]
             # calculate two orthogonal vectors to constraint (strike and dip vector)
-            strike_vector, dip_vector = get_vectors(points[inside, 3:6])
+            strike_vector, dip_vector = get_vectors(
+                points[
+                    inside, self.support.dimension : self.support.dimension + self.support.dimension
+                ]
+            )
             A = np.einsum("ij,ijk->ik", strike_vector.T, T)
             B = np.zeros(points[inside, :].shape[0])
             self.add_constraints_to_least_squares(A, B, idc[inside, :], w=w, name="gradient")
             A = np.einsum("ij,ijk->ik", dip_vector.T, T)
             self.add_constraints_to_least_squares(A, B, idc[inside, :], w=w, name="gradient")
+            # self.regularisation_scale += compute_weighting(
+            #     self.support.nodes,
+            #     points[inside, : self.support.dimension],
+            #     sigma=self.support.nsteps[0] * 10,
+            # )
             if np.sum(inside) <= 0:
                 logger.warning(
                     f" {np.sum(~inside)} \
@@ -302,10 +326,12 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         if points.shape[0] > 0:
             # calculate unit vector for orientation data
             # points[:,3:]/=np.linalg.norm(points[:,3:],axis=1)[:,None]
-            node_idx, inside = self.support.position_to_cell_corners(points[:, :3])
+            node_idx, inside = self.support.position_to_cell_corners(
+                points[:, : self.support.dimension]
+            )
             gi = np.zeros(self.support.n_nodes)
             gi[:] = -1
-            gi[self.region] = np.arange(0, self.nx)
+            gi[self.region] = np.arange(0, self.dof)
             idc = np.zeros(node_idx.shape)
             idc[:] = -1
             idc[inside, :] = gi[node_idx[inside, :]]
@@ -319,31 +345,40 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
                 T,
                 elements,
                 inside_,
-            ) = self.support.get_element_gradient_for_location(points[inside, :3])
+            ) = self.support.get_element_gradient_for_location(
+                points[inside, : self.support.dimension]
+            )
             # T*=np.product(self.support.step_vector)
             # T/=self.support.step_vector[0]
+            # indexes, inside2 = self.support.position_to_nearby_cell_indexes(
+            # points[inside, : self.support.dimension]
+            # )
+            # indexes = indexes[inside2, :]
+
+            # corners = self.support.cell_corner_indexes(indexes)
+            # node_indexes = corners.reshape(-1, 3)
+            # indexes = self.support.global_node_indices(indexes)
+            # self.regularisation_scale[indexes]  =10
+
+            self.regularisation_scale += compute_weighting(
+                self.support.nodes,
+                points[inside, : self.support.dimension],
+                sigma=self.support.nsteps[0] * 10,
+            )
+            # global_indexes = self.support.neighbour_global_indexes().T.astype(int)
+            # close_indexes =
+            # self.regularisation_scale[global_indexes[idc[inside,:].astype(int),]]=10
             w /= 3
-            self.add_constraints_to_least_squares(
-                T[:, 0, :],
-                points[inside, 3],
-                idc[inside, :],
-                w=w,
-                name="norm",
-            )
-            self.add_constraints_to_least_squares(
-                T[:, 1, :],
-                points[inside, 4],
-                idc[inside, :],
-                w=w,
-                name="norm",
-            )
-            self.add_constraints_to_least_squares(
-                T[:, 2, :],
-                points[inside, 5],
-                idc[inside, :],
-                w=w,
-                name="norm",
-            )
+            for d in range(self.support.dimension):
+
+                self.add_constraints_to_least_squares(
+                    T[:, d, :],
+                    points[inside, self.support.dimension + d],
+                    idc[inside, :],
+                    w=w,
+                    name=f"norm_{d}",
+                )
+
             if np.sum(inside) <= 0:
                 logger.warning(
                     f"{np.sum(~inside)} \
@@ -356,7 +391,7 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         points: np.ndarray,
         vectors: np.ndarray,
         w: float = 1.0,
-        B: float = 0,
+        b: float = 0,
         name="gradient orthogonal",
     ):
         """
@@ -378,13 +413,15 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
         if points.shape[0] > 0:
 
             # calculate unit vector for orientation data
-            node_idx, inside = self.support.position_to_cell_corners(points[:, :3])
+            node_idx, inside = self.support.position_to_cell_corners(
+                points[:, : self.support.dimension]
+            )
             # calculate unit vector for node gradients
             # this means we are only constraining direction of grad not the
             # magnitude
             gi = np.zeros(self.support.n_nodes)
             gi[:] = -1
-            gi[self.region] = np.arange(0, self.nx)
+            gi[self.region] = np.arange(0, self.dof)
             idc = np.zeros(node_idx.shape)
             idc[:] = -1
 
@@ -400,13 +437,15 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
                 T,
                 elements,
                 inside_,
-            ) = self.support.get_element_gradient_for_location(points[inside, :3])
+            ) = self.support.get_element_gradient_for_location(
+                points[inside, : self.support.dimension]
+            )
             T[norm > 0, :, :] /= norm[norm > 0, None, None]
 
             # dot product of vector and element gradient = 0
-            A = np.einsum("ij,ijk->ik", vectors[inside, :3], T)
-            B = np.zeros(points[inside, :].shape[0]) + B
-            self.add_constraints_to_least_squares(A, B, idc[inside, :], w=w, name=name)
+            A = np.einsum("ij,ijk->ik", vectors[inside, : self.support.dimension], T)
+            b_ = np.zeros(points[inside, :].shape[0]) + b
+            self.add_constraints_to_least_squares(A, b_, idc[inside, :], w=w, name=name)
 
             if np.sum(inside) <= 0:
                 logger.warning(
@@ -415,24 +454,30 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
                 )
             self.up_to_date = False
 
-    def add_regularisation(self, operator, w=0.1):
-        """
 
-        Parameters
-        ----------
-        operator
-        w
 
-        Returns
-        -------
+    # def assemble_borders(self, operator, w, name='regularisation'):
+    #     """
+    #     Adds a constraint to the border of the model to force the value to be equal to the value at the border
 
-        """
-        self.assemble_inner(operator, w)
-        # self.assemble_borders()
+    #     Parameters
+    #     ----------
+    #     operator : Operator
+    #         operator to use for the regularisation
+    #     w : double
+    #         weight of the regularisation
 
-    # def assemble_borders(self, operator, w):
+    #     Returns
+    #     -------
 
-    def assemble_inner(self, operator, w):
+    #     """
+    #     # First get the global indicies of the pairs of neighbours this should be an
+    #     # N*27 array for 3d and an N*9 array for 2d
+
+    #     global_indexes = self.support.neighbour_global_indexes()
+
+
+    def assemble_inner(self, operator, w, name='regularisation'):
         """
 
         Parameters
@@ -445,7 +490,7 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
 
         """
         # First get the global indicies of the pairs of neighbours this should be an
-        # Nx27 array for 3d and an Nx9 array for 2d
+        # N*27 array for 3d and an N*9 array for 2d
 
         global_indexes = self.support.neighbour_global_indexes()  # np.array([ii,jj]))
 
@@ -454,7 +499,7 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
 
         gi = np.zeros(self.support.n_nodes)
         gi[:] = -1
-        gi[self.region] = np.arange(0, self.nx)
+        gi[self.region] = np.arange(0, self.dof)
         idc = gi[idc]
         inside = ~np.any(idc == -1, axis=1)  # np.ones(a.shape[0],dtype=bool)#
         # a[idc==-1] = 0
@@ -464,7 +509,11 @@ class FiniteDifferenceInterpolator(DiscreteInterpolator):
             a[inside, :],
             B[inside],
             idc[inside, :],
-            w=w,
-            name="regularisation",
+            w=(
+                self.regularisation_scale[idc[inside, 13].astype(int)] * w
+                if self.use_regularisation_weight_scale
+                else w
+            ),
+            name=name,
         )
         return
