@@ -6,12 +6,12 @@ Geological modelling classes and functions
 __all__ = [
     "GeologicalModel",
     "ProcessInputData",
-    "Loop3DView",
     "Map2LoopProcessor",
     "LoopProjectfileProcessor",
 ]
 from ..utils import getLogger
 from ..utils import LoopImportError
+from .core.geological_model import GeologicalModel
 
 logger = getLogger(__name__)
 from ..modelling.input import (
@@ -22,8 +22,14 @@ from ..modelling.input import (
 try:
     from ..modelling.input.project_file import LoopProjectfileProcessor
 except (LoopImportError, ImportError):
-    logger.warning(
-        "Cannot use LoopProjectfileProcessor: Loop project file cannot be imported, try installing LoopProjectFile"
-    )
-# from LoopStructural.modelling.features import StructuralFrame
-# from LoopStructural.modelling.features.fault import FaultSegment
+    class LoopProjectfileProcessor(ProcessInputData):
+        """
+        Dummy class to handle the case where LoopProjectFile is not installed.
+        This will raise a warning when used.
+        """
+
+        def __init__(self, *args, **kwargs):
+            raise LoopImportError(
+                "LoopProjectFile cannot be imported. Please install LoopProjectFile."
+            )
+    
