@@ -25,7 +25,6 @@ from ....modelling.features.builders import BaseBuilder
 from ....utils.helper import (
     get_data_bounding_box_map as get_data_bounding_box,
 )
-from ....utils import RegionEverywhere
 from ....interpolators import DiscreteInterpolator
 from ....interpolators import InterpolatorFactory
 
@@ -39,7 +38,6 @@ class GeologicalFeatureBuilder(BaseBuilder):
         bounding_box,
         nelements: int = 1000,
         name="Feature",
-        interpolation_region=None,
         model=None,
         **kwargs,
     ):
@@ -61,13 +59,13 @@ class GeologicalFeatureBuilder(BaseBuilder):
             nelements=nelements,
             buffer=kwargs.get("buffer", 0.2),
         )
-
+        
         if not issubclass(type(interpolator), GeologicalInterpolator):
             raise TypeError(
                 "interpolator is {} and must be a GeologicalInterpolator".format(type(interpolator))
             )
         self._interpolator = interpolator
-
+        self._up_to_date = self._interpolator.up_to_date
         header = (
             xyz_names()
             + val_name()
@@ -78,38 +76,34 @@ class GeologicalFeatureBuilder(BaseBuilder):
         )
         self.data = pd.DataFrame(columns=header)
         self.data_added = False
-        self._interpolation_region = None
-        self.interpolation_region = interpolation_region
-        if self.interpolation_region is not None:
-            self._interpolator.set_region(region=self.interpolation_region)
 
         self._feature = GeologicalFeature(
             self._name,
-            self._interpolator,
             builder=self,
             regions=[],
             faults=self.faults,
         )
         self._orthogonal_features = {}
         self._equality_constraints = {}
+        # add default parameters
+        self.update_build_arguments({'cpw':1.0,'npw':1.0,'regularisation':.10,'nelements':self.interpolator.n_elements})
+    def set_not_up_to_date(self, caller):
+        logger.info(
+            f"Setting {self.name} to not up to date from an instance of {caller.__class__.__name__}"
+        )
+        self._up_to_date = False
+        self._interpolator.up_to_date = False
 
     @property
     def interpolator(self):
         return self._interpolator
 
-    @property
-    def interpolation_region(self):
-        return self._interpolation_region
-
-    @interpolation_region.setter
-    def interpolation_region(self, interpolation_region):
-        if interpolation_region is not None:
-            self._interpolation_region = interpolation_region
-            self._interpolator.set_region(region=self._interpolation_region)
-        else:
-            self._interpolation_region = RegionEverywhere()
-            self._interpolator.set_region(region=self._interpolation_region)
-        self._up_to_date = False
+    @interpolator.setter
+    def interpolator(self, interpolator):
+        if not issubclass(type(interpolator), GeologicalInterpolator):
+            raise TypeError(
+                "interpolator is {} and must be a GeologicalInterpolator".format(type(interpolator))
+            )
 
     def add_data_from_data_frame(self, data_frame, overwrite=False):
         """
@@ -156,6 +150,8 @@ class GeologicalFeatureBuilder(BaseBuilder):
             logger.error("Cannot cast {} as integer, setting step to 1".format(step))
             step = 1
         self._orthogonal_features[feature.name] = [feature, w, region, step, B]
+
+        logger.info(f"Adding orthogonal constraint {feature.name} to {self.name}")
         self._up_to_date = False
 
     def add_data_to_interpolator(self, constrained=False, force_constrained=False, **kwargs):
@@ -173,6 +169,7 @@ class GeologicalFeatureBuilder(BaseBuilder):
 
         """
         if self.data_added:
+            logger.info("Data already added to interpolator")
             return
         # first move the data for the fault
         logger.info(f"Adding {len(self.faults)} faults to {self.name}")
@@ -315,12 +312,13 @@ class GeologicalFeatureBuilder(BaseBuilder):
                     self.interpolator.support.barycentre[element_idx[::step], :],
                     vector[element_idx[::step], :],
                     w=w,
-                    B=B,
+                    b=B,
                     name=f'{feature.name}_orthogonal',
                 )
 
     def add_equality_constraints(self, feature, region, scalefactor=1.0):
         self._equality_constraints[feature.name] = [feature, region, scalefactor]
+        logger.info(f'Adding equality constraints to {self.name}')
         self._up_to_date = False
 
     def install_equality_constraints(self):
@@ -454,7 +452,7 @@ class GeologicalFeatureBuilder(BaseBuilder):
         self.interpolator.support.rotation_xy = rotation
         self._up_to_date = False
 
-        while self.interpolator.nx < 100:
+        while self.interpolator.dof < 100:
             self.interpolator.support.step_vector = self.interpolator.support.step_vector * 0.9
 
     def check_interpolation_geometry(self, data, buffer=0.3):
@@ -463,19 +461,22 @@ class GeologicalFeatureBuilder(BaseBuilder):
         Apply the fault to the model grid to ensure that the support
         is big enough to capture the faulted feature.
         """
+        if self.interpolator.support is not None:
+            logger.info(f"Checking interpolation geometry for {self.name}")
+            origin = self.interpolator.support.origin
+            maximum = self.interpolator.support.maximum
+            pts = self.model.bounding_box.with_buffer(buffer).regular_grid(local=True)
+            for f in self.faults:
+                pts = f.apply_to_points(pts)
 
-        origin = self.interpolator.support.origin
-        maximum = self.interpolator.support.maximum
-        pts = self.model.bounding_box.with_buffer(buffer).regular_grid(local=True)
-        for f in self.faults:
-            pts = f.apply_to_points(pts)
-
-        origin[origin > np.min(pts, axis=0)] = np.min(pts, axis=0)[origin > np.min(pts, axis=0)]
-        maximum[maximum < np.max(pts, axis=0)] = np.max(pts, axis=0)[maximum < np.max(pts, axis=0)]
-        self.interpolator.support.origin = origin
-        self.interpolator.support.maximum = maximum
-
-    def build(self, fold=None, fold_weights={}, data_region=None, **kwargs):
+            origin[origin > np.min(pts, axis=0)] = np.min(pts, axis=0)[origin > np.min(pts, axis=0)]
+            maximum[maximum < np.max(pts, axis=0)] = np.max(pts, axis=0)[
+                maximum < np.max(pts, axis=0)
+            ]
+            self.interpolator.support.origin = origin
+            self.interpolator.support.maximum = maximum
+            self.update_build_arguments({'nelements':self.interpolator.n_elements})
+    def build(self, data_region=None, **kwargs):
         """
         Runs the interpolation and builds the geological feature
 
@@ -497,6 +498,15 @@ class GeologicalFeatureBuilder(BaseBuilder):
         for f in self.faults:
             f.builder.update()
         domain = kwargs.get("domain", None)
+        if 'nelements' in kwargs:
+            # if the number of elements has changed then update the interpolator
+            logger.info(f'Interpolator has {self.interpolator.n_elements} elements')
+            logger.info(f'Kwargs has {kwargs["nelements"]} elements')
+            if self.interpolator.n_elements != kwargs['nelements']:
+                logger.info('Setting nelements to {} for {}'.format(kwargs['nelements'], self.name))
+                self.build_arguments['nelements'] = self.interpolator.set_nelements(kwargs['nelements'])
+                logger.info(f'Interpolator nelements {self.interpolator.n_elements}')
+                self._up_to_date = False
         if domain:
             self.check_interpolation_geometry(None)
         self.add_data_to_interpolator(**kwargs)
@@ -536,7 +546,9 @@ class GeologicalFeatureBuilder(BaseBuilder):
         logger.info(f'running interpolation for {self.name}')
 
         self.interpolator.solve_system(
-            solver=kwargs.get('solver', None), solver_kwargs=kwargs.get('solver_kwargs', {})
+            solver=kwargs.get('solver', 'cg'),
+            tol=kwargs.get('tol', None),
+            solver_kwargs=kwargs.get('solver_kwargs', {}),
         )
         logger.info(f'Finished building  {self.name}')
         self._up_to_date = True
