@@ -1,8 +1,9 @@
+from LoopStructural.utils.maths import regular_tetraherdron_for_points, gradient_from_tetrahedron
 from ....modelling.features.fault._fault_function_feature import (
     FaultDisplacementFeature,
 )
 from ....modelling.features import FeatureType
-from ....modelling.features.fault._fault_function import BaseFault, BaseFault3D
+from ....modelling.features.fault._fault_function import BaseFault, BaseFault3D, FaultDisplacement
 from ....utils import getLogger, NegativeRegion, PositiveRegion
 from ....modelling.features import StructuralFrame
 
@@ -21,7 +22,14 @@ class FaultSegment(StructuralFrame):
     """
 
     def __init__(
-        self, features, name, faultfunction=None, steps=10, displacement=1.0, fold=None, model=None
+        self,
+        name: str,
+        features: list,
+        faultfunction=None,
+        steps=10,
+        displacement=1.0,
+        fold=None,
+        model=None,
     ):
         """
         A slip event of a fault
@@ -36,10 +44,10 @@ class FaultSegment(StructuralFrame):
             how many integration steps for faults
         kwargs
         """
-        StructuralFrame.__init__(self, features, name, fold, model)
+        StructuralFrame.__init__(self, name=name, features=features, fold=fold, model=model)
         self.type = FeatureType.FAULT
         self.displacement = displacement
-        self._faultfunction = BaseFault.fault_displacement
+        self._faultfunction = BaseFault().fault_displacement
         self.steps = steps
         self.regions = []
         self.faults_enabled = True
@@ -57,10 +65,13 @@ class FaultSegment(StructuralFrame):
     def faultfunction(self, value):
         if callable(value):
             self._faultfunction = value
+        if issubclass(FaultDisplacement, type(value)):
+            self._faultfunction = value
+
         elif isinstance(value, str) and value == "BaseFault":
-            self._faultfunction = BaseFault.fault_displacement
+            self._faultfunction = BaseFault().fault_displacement
         elif isinstance(value, str) and value == "BaseFault3D":
-            self._faultfunction = BaseFault3D.fault_displacement
+            self._faultfunction = BaseFault3D().fault_displacement
         else:
             raise ValueError("Fault function must be a function or BaseFault")
 
@@ -93,7 +104,7 @@ class FaultSegment(StructuralFrame):
         if self.builder is None:
             raise ValueError("Fault builder not set")
         return self.builder.fault_major_axis
-
+    
     @property
     def fault_intermediate_axis(self):
         if self.builder is None:
@@ -110,6 +121,32 @@ class FaultSegment(StructuralFrame):
     def displacementfeature(self):
         return FaultDisplacementFeature(self, self.faultfunction, name=self.name, model=self.model)
 
+    def fault_ellipsoid(self, **kwargs):
+        try:
+            import pyvista as pv
+
+            if self.model is None:
+                pts = self.fault_centre[None, :]
+            else:
+                pts = self.model.rescale(self.fault_centre[None, :], inplace=False)
+            # pts = self.fault_centre[None, :]
+            fault_ellipsoid = pv.PolyData(pts)
+            # fault_ellipsoid = pv.PolyData(
+            #     self.model.rescale(self.fault_centre[None, :], inplace=False)
+            # )
+            fault_ellipsoid["norm"] = self.fault_normal_vector[None, :]
+            fault_ellipsoid['norm'] /= np.linalg.norm(fault_ellipsoid['norm'], axis=1)[:, None]
+            geom = pv.ParametricEllipsoid(
+                self.fault_minor_axis,
+                self.fault_major_axis,
+                self.fault_intermediate_axis,
+            )
+            ellipsoid = fault_ellipsoid.glyph(geom=geom, **kwargs)
+            return ellipsoid
+        except ImportError:
+            logger.error("pyvista not installed")
+            return None
+
     def set_fault_offset(self, offset: float):
         self.fault_offset = offset
 
@@ -173,7 +210,7 @@ class FaultSegment(StructuralFrame):
         return np.abs(v) > threshold
         # return np.all(np.logical_and(v > -1,v<1),axis=1)
 
-    def evaluate_value(self, locations):
+    def evaluate_value(self, locations, ignore_regions=False):
         """
         Return the value of the fault surface scalar field
 
@@ -198,7 +235,10 @@ class FaultSegment(StructuralFrame):
         #     except:
         #         logger.error("nan slicing")
         # v[mask] = self.__getitem__(0).evaluate_value(locations[mask, :])
-        return self.__getitem__(0).evaluate_value(locations)
+        return self.__getitem__(0).evaluate_value(locations, ignore_regions=ignore_regions)
+
+    def ellipsoid(self):
+        pass
 
     def mean(self):
         return self.__getitem__(0).mean()
@@ -234,8 +274,9 @@ class FaultSegment(StructuralFrame):
                 logger.error("nan slicing ")
         # need to scale with fault displacement
         v[mask, :] = self.__getitem__(1).evaluate_gradient(locations[mask, :])
+        v[mask, :] /= np.linalg.norm(v[mask, :], axis=1)[:, None]
         scale = self.displacementfeature.evaluate_value(locations[mask, :])
-        v[mask, :] *= scale[:, None]
+        v[mask, :] *= scale[:, None] * self.displacement
         return v
 
     def evaluate_displacement(self, points):
@@ -282,6 +323,7 @@ class FaultSegment(StructuralFrame):
         -------
 
         """
+        logger.info(f'Applying fault {self.name} to points {points.shape}')
         steps = self.steps
         newp = np.copy(points).astype(float)
         # evaluate fault function for all points
@@ -389,51 +431,12 @@ class FaultSegment(StructuralFrame):
         # the nodes of the tetrahedron are then restored by the fault and then gradient is
         # recalculated using the updated node positions but the original corner values
 
-        regular_tetrahedron = np.array(
-            [
-                [np.sqrt(8 / 9), 0, -1 / 3],
-                [-np.sqrt(2 / 9), np.sqrt(2 / 3), -1 / 3],
-                [-np.sqrt(2 / 9), -np.sqrt(2 / 3), -1 / 3],
-                [0, 0, 1],
-            ]
-        )
-        regular_tetrahedron *= scale_parameter
-        xyz = vector[:, :3]
-        tetrahedron = np.zeros((xyz.shape[0], 4, 3))
-        tetrahedron[:] = xyz[:, None, :]
-        tetrahedron[:, :, :] += regular_tetrahedron[None, :, :]
+        tetrahedron = regular_tetraherdron_for_points(vector[:, :3], scale_parameter)
+        corners = np.einsum('ikj,ij->ik', tetrahedron - vector[:, None, :3], vector[:, 3:])
 
-        vectors = vector[:, 3:]
-        corners = np.einsum('ikj,ij->ik', tetrahedron - xyz[:, None, :], vectors)
-        tetrahedron = tetrahedron.reshape(-1, 3)
-        tetrahedron = self.apply_to_points(tetrahedron)
-        tetrahedron = tetrahedron.reshape(-1, 4, 3)
-        m = np.array(
-            [
-                [
-                    (tetrahedron[:, 1, 0] - tetrahedron[:, 0, 0]),
-                    (tetrahedron[:, 1, 1] - tetrahedron[:, 0, 1]),
-                    (tetrahedron[:, 1, 2] - tetrahedron[:, 0, 2]),
-                ],
-                [
-                    (tetrahedron[:, 2, 0] - tetrahedron[:, 0, 0]),
-                    (tetrahedron[:, 2, 1] - tetrahedron[:, 0, 1]),
-                    (tetrahedron[:, 2, 2] - tetrahedron[:, 0, 2]),
-                ],
-                [
-                    (tetrahedron[:, 3, 0] - tetrahedron[:, 0, 0]),
-                    (tetrahedron[:, 3, 1] - tetrahedron[:, 0, 1]),
-                    (tetrahedron[:, 3, 2] - tetrahedron[:, 0, 2]),
-                ],
-            ]
-        )
-        I = np.array([[-1.0, 1.0, 0.0, 0.0], [-1.0, 0.0, 1.0, 0.0], [-1.0, 0.0, 0.0, 1.0]])
-        m = np.swapaxes(m, 0, 2)
-        element_gradients = np.linalg.inv(m)
+        tetrahedron = self.apply_to_points(tetrahedron.reshape(-1, 3)).reshape(-1, 4, 3)
+        v = gradient_from_tetrahedron(tetrahedron, corners)
 
-        element_gradients = element_gradients.swapaxes(1, 2)
-        element_gradients = element_gradients @ I
-        v = np.sum(element_gradients * corners[:, None, :], axis=2)
         return v
 
     def add_abutting_fault(self, abutting_fault_feature, positive=None):
@@ -464,7 +467,7 @@ class FaultSegment(StructuralFrame):
             )
         self.abut[abutting_fault_feature.name] = abutting_region
         self.__getitem__(0).add_region(abutting_region)
-
+        return abutting_region
     def save(self, filename, scalar_field=True, slip_vector=True, surface=True):
         """
         Save the fault to a file
