@@ -1,6 +1,4 @@
-import numpy as np
 from ....modelling.features import BaseFeature
-
 from ....utils import getLogger
 
 logger = getLogger(__name__)
@@ -44,24 +42,3 @@ class FoldRotationAngleFeature(BaseFeature):
         s1 = self.fold_frame.features[0].evaluate_value(location)
         r = self.rotation(s1)
         return r
-
-
-def fourier_series(x, c0, c1, c2, w):
-    """
-
-    Parameters
-    ----------
-    x
-    c0
-    c1
-    c2
-    w
-
-    Returns
-    -------
-
-    """
-    v = np.array(x.astype(float))
-    # v.fill(c0)
-    v = c0 + c1 * np.cos(2 * np.pi / w * x) + c2 * np.sin(2 * np.pi / w * x)
-    return np.rad2deg(np.arctan(v))
