@@ -78,7 +78,9 @@ class ProcessInputData:
         self.contacts = contacts
         self._contact_orientations = None
         self.contact_orientations = contact_orientations
-        self._fault_orientations = None
+        self._fault_orientations = pd.DataFrame(
+            columns=["X", "Y", "Z", "gx", "gy", "gz", "coord", "feature_name"]
+        )
         self.fault_orientations = fault_orientations
         self._fault_locations = None
         self.fault_locations = fault_locations
@@ -165,8 +167,19 @@ class ProcessInputData:
             unit_id = 1
             val = self._stratigraphic_value()
             for name, sg in self._stratigraphic_order:
+                # set the oldest unit to be the basement.
+                # this has no observed basal contact and then
+                # top of the unit is the 0 isovalue.
+                # this is the minimum of the next unit
                 stratigraphic_column[name] = {}
-                for i, g in enumerate(reversed(sg)):
+                stratigraphic_column[name][sg[-1]] = {
+                    "max": 0,
+                    "min": -np.inf,
+                    "id": unit_id,
+                    "colour": self.colours[sg[-1]],
+                }
+                # iterate through the remaining units (in reverse)
+                for g in reversed(sg[:-1]):
                     if g in self.thicknesses:
                         stratigraphic_column[name][g] = {
                             "max": val[g] + self.thicknesses[g],
@@ -174,10 +187,6 @@ class ProcessInputData:
                             "id": unit_id,
                             "colour": self.colours[g],
                         }
-                        if i == 0:
-                            stratigraphic_column[name][g]["min"] = 0
-                        if i == len(sg) - 1:
-                            stratigraphic_column[name][g]["max"] = np.inf
 
                     unit_id += 1
             # add faults into the column
@@ -306,10 +315,11 @@ class ProcessInputData:
                 pts = self.fault_orientations.loc[
                     self.fault_orientations["feature_name"] == fname, ["gx", "gy", "gz"]
                 ]
-                fault_properties.loc[
-                    fname,
-                    ["avgNormalEasting", "avgNormalNorthing", "avgNormalAltitude"],
-                ] = np.nanmean(pts, axis=0)
+                if len(pts) > 0:
+                    fault_properties.loc[
+                        fname,
+                        ["avgNormalEasting", "avgNormalNorthing", "avgNormalAltitude"],
+                    ] = np.nanmean(pts, axis=0)
         if (
             "avgSlipDirEasting" not in fault_properties.columns
             or "avgSlipDirNorthing" not in fault_properties.columns
@@ -392,7 +402,7 @@ class ProcessInputData:
             dataframes.append(self.contacts)
         if self.contact_orientations is not None:
             dataframes.append(self.contact_orientations)
-        if self.fault_orientations is not None:
+        if len(self.fault_orientations) > 0:
             dataframes.append(self.fault_orientations)
         if self.fault_locations is not None:
             dataframes.append(self.fault_locations)
@@ -438,9 +448,22 @@ class ProcessInputData:
         stratigraphic_value = {}
         for _name, sg in self.stratigraphic_order:
             value = 0.0  # reset for each supergroup
-            for g in reversed(sg):
+            if sg[0] not in self.thicknesses or self.thicknesses[sg[0]] <= 0:
+                self.thicknesses[sg[0]] = (
+                    np.inf
+                )  # make the top unit infinite as it should extend to the top of the model
+            for g in reversed(
+                sg[:-1]
+            ):  # don't add the last unit as we never see the base of this unit.
+                # It should be "basement"
                 if g not in self.thicknesses:
                     logger.warning(f"No thicknesses for {g}")
+                    stratigraphic_value[g] = np.nan
+                if self.thicknesses[g] <= 0:
+                    logger.error(
+                        f"Thickness for {g} is less than or equal to 0\n Update the thickness value for {g} before continuing"
+                    )
+
                     stratigraphic_value[g] = np.nan
                 else:
                     stratigraphic_value[g] = value
@@ -463,7 +486,25 @@ class ProcessInputData:
 
     @property
     def contacts(self):
-        return self._contacts
+        contacts = self._contacts.copy()
+        if self._use_thickness:
+            contacts["val"] = np.nan
+            for k, v in self._stratigraphic_value().items():
+                contacts.loc[contacts["name"] == k, "val"] = v
+
+            contacts = contacts.loc[
+                ~np.isnan(contacts["val"]), ["X", "Y", "Z", "feature_name", "val"]
+            ]
+        if not self._use_thickness:
+            contacts["interface"] = np.nan
+            interface_val = 0
+            for k in self._stratigraphic_value().keys():
+                contacts.loc[contacts["name"] == k, "interface"] = interface_val
+            contacts = contacts.loc[
+                ~np.isnan(contacts["interface"]),
+                ["X", "Y", "Z", "feature_name", "interface"],
+            ]
+        return contacts
 
     @contacts.setter
     def contacts(self, contacts):
@@ -479,25 +520,8 @@ class ProcessInputData:
         """
         if contacts is None:
             return
-        contacts = contacts.copy()
-        self._update_feature_names(contacts)
-        if self._use_thickness:
-            contacts["val"] = np.nan
-            for k, v in self._stratigraphic_value().items():
-                contacts.loc[contacts["name"] == k, "val"] = v
-
-            self._contacts = contacts.loc[
-                ~np.isnan(contacts["val"]), ["X", "Y", "Z", "feature_name", "val"]
-            ]
-        if not self._use_thickness:
-            contacts["interface"] = np.nan
-            interface_val = 0
-            for k in self._stratigraphic_value().keys():
-                contacts.loc[contacts["name"] == k, "interface"] = interface_val
-            self._contacts = contacts.loc[
-                ~np.isnan(contacts["interface"]),
-                ["X", "Y", "Z", "feature_name", "interface"],
-            ]
+        self._contacts = contacts.copy()
+        self._update_feature_names(self._contacts)
 
     @property
     def contact_orientations(self):
