@@ -23,6 +23,7 @@ from .maths import (
     get_strike_vector,
     get_vectors,
     strikedip2vector,
+    plungeazimuth2vector,
     azimuthplunge2vector,
     normal_vector_to_strike_and_dip,
     rotate,
@@ -36,3 +37,4 @@ import numpy as np
 rng = np.random.default_rng()
 
 from ._surface import LoopIsosurfacer, surface_list
+from .colours import random_colour, random_hex_colour
